@@ -49,6 +49,7 @@
 #include <csignal>
 #include <cstdio>
 #include <ctime>
+#include <cfloat>
 
 #ifdef FC_OS_WIN32
 # include <crtdbg.h>
@@ -68,6 +69,7 @@
 
 // STL
 #include <bitset>
+#include <chrono>
 #include <exception>
 #include <functional>
 #include <iterator>
@@ -98,6 +100,8 @@
 #include <boost/filesystem/operations.hpp>
 #include <boost/filesystem/path.hpp>
 
-#endif //_PreComp_
+#include <fmt/format.h>
+
+#endif  //_PreComp_
 
 #endif // APP_PRECOMPILED_H
